@@ -14,7 +14,7 @@ use File::Spec ();
 
 use vars qw{$VERSION $ISCORE @ISA};
 BEGIN {
-	$VERSION = '0.64';
+	$VERSION = '0.67';
 	$ISCORE  = 1;
 	@ISA     = qw{Module::Install::Base};
 }
@@ -24,10 +24,40 @@ BEGIN {
 
 
 #####################################################################
-# CPAN Clients
+# CPAN Client
 
-# Are we currently running under the CPAN.pm client
+# What CPAN client are we running under
+sub cpan {
+	return 'cpanpm'   if cpanpm();
+	return 'cpanplus' if cpanplus();
+	return 'legacy'   if cpanlegacy();
+}
+
+# Are we running a legacy client
+sub cpanlegacy {
+	return 1 if cpanpm_legacy();
+	return 1 if cpanplus_legacy();
+	return '';
+}
+
+# Are we currently running under a CPAN.pm client (modern method)
+# Returns the version of the CPAN.pm client.
 sub cpanpm {
+	my $script = File::Spec->rel2abs($0);
+	return (
+		$ENV{PERL5_CPAN_IS_EXECUTING}
+		and
+		-f $script
+		and
+		$ENV{PERL5_CPAN_IS_EXECUTING} eq $script
+	)
+		? ($ENV{PERL5_CPAN_IS_VERSION} || -1)
+		: '';
+}
+
+# Are we currently running under the legacy CPAN client
+# This method is not reliable.
+sub cpanpm_legacy {
 	my $self = shift;
 
 	# Does the lock file exist?
@@ -70,7 +100,14 @@ sub cpanpm_config {
 
 # Are we currently running under the CPANPLUS client
 sub cpanplus {
-	!! $ENV{PERL5_CPANPLUS_IS_RUNNING};
+	my $script = File::Spec->rel2abs($0);
+	return (
+		$ENV{PERL5_CPANPLUS_IS_EXECUTING}
+		and
+		-f $script
+		and
+		$ENV{PERL5_CPANPLUS_IS_EXECUTING} eq $script
+	) ? ($ENV{PERL5_CPANPLUS_IS_VERSION} || -1) : '';
 }
 
 # Is CPANPLUS actually installed
@@ -78,23 +115,9 @@ sub cpanplus_available {
 	$_[0]->can_use('CPANPLUS');
 }
 
-# Get a cpanplus configuration key.
-# Assumes you have already check cpanplus is at least installed.
-sub cpanplus_config {
-	my $self = shift;
-
-	# Load the CPANPLUS configuration
-	require CPANPLUS::Configure;
-	my $conf = CPANPLUS::Configure->new
-		or die "Failed to load CPANPLUS configuration";
-
-	# Get the configuration key
-	return $conf->get_conf(shift);
-}
-
-# Are we currently running under some automated testing system
-sub automated_testing {
-	!! $ENV{AUTOMATED_TESTING};
+# Are we (maybe) running under a legacy version of CPANPLUS
+sub cpanplus_legacy {
+	!! $ENV{CPANPLUS_IS_RUNNING};
 }
 
 
@@ -129,14 +152,25 @@ sub no_mb {
 
 
 #####################################################################
-# User vs Author Context
+# Testing and Configuration Contexts
 
-sub user_mode {
-	1;
+# Are we in an interactive configuration environment
+sub interactive {
+	# Treat things interactively ONLY based on input
+	!! -t STDIN;
 }
 
-sub author_mode {
-	1;
+# Are we currently running under some sort of automated testing system
+sub automated_testing {
+	!! $ENV{AUTOMATED_TESTING};
+}
+
+sub user_context {
+	! $Module::Install::AUTHOR;
+}
+
+sub author_context {
+	!! $Module::Install::AUTHOR;
 }
 
 1;
